@@ -474,7 +474,7 @@ def SmartSeedPredictionSliced(SaveDir, fname, UnetModel, StarModel, NoiseModel =
     
     
 def SmartSeedPrediction3D( SaveDir, fname,  UnetModel, StarModel, NoiseModel = None, min_size_mask = 100, min_size = 100, max_size = 100000,
-n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projection = False, UseProbability = True, filtersize = 0, globalthreshold = 1.0E-5, extent = 0, seedpool = True):
+n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projection = False, UseProbability = True, filtersize = 0, globalthreshold = 1.0E-5, extent = 0, dounet = True, seedpool = True, otsu = False, startZ = 0):
     
     
     
@@ -495,6 +495,7 @@ n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projec
     Path(MarkerResults).mkdir(exist_ok = True)
     #Read Image
     image = imread(fname)
+    
     sizeZ = image.shape[0]
     sizeY = image.shape[1]
     sizeX = image.shape[2]
@@ -505,23 +506,50 @@ n_tiles = (1,2,2), doMask = True, smartcorrection = None, threshold = 20, projec
     Name = os.path.basename(os.path.splitext(fname)[0])
     if NoiseModel is not None:
          print('Denoising Image')
+        
          image = NoiseModel.predict(image, axes='ZYX', n_tiles=n_tiles)
+         newimage = np.zeros(image.shape, dtype = 'float32')
+         if otsu:
+             for i in range(0, image.shape[0]): 
+                 thresh = threshold_otsu(image[i,:])
+                 image[i,:][image[i,:] <= thresh] = 0
+                 newimage[i,:] = image[i,:]
+             image = newimage  
          imwrite((DenoiseResults + Name+ '.tif' ) , image.astype('float32'))   
-    print('UNET segmentation on Image')     
-    Mask = UNETPrediction3D(gaussian_filter(image, filtersize), UnetModel, n_tiles, 'ZYX')
-    for i in range(0, Mask.shape[0]):
-        Mask[i,:] = remove_small_objects(Mask[i,:].astype('uint16'), min_size = min_size)
     
+    if dounet:
+        print('UNET segmentation on Image')     
+
+        Mask = UNETPrediction3D(gaussian_filter(image, filtersize), UnetModel, n_tiles, 'ZYX')
+    else:
+      
+      Mask = np.zeros(image.shape)
+      
+      for i in range(0, Mask.shape[0]):
+
+                 thresh = threshold_otsu(image[i,:])
+                 Mask[i,:] = image[i,:] > thresh  
+                 Mask[i,:] = label(Mask[i,:]) 
+    
+                 Mask[i,:] = remove_small_objects(Mask[i,:].astype('uint16'), min_size = min_size)
+                 Mask[i,:] = remove_big_objects(Mask[i,:].astype('uint16'), max_size = max_size)
+    
+    Mask = label(Mask > 0)       
     SizedMask[:, :Mask.shape[1], :Mask.shape[2]] = Mask
     imwrite((UNETResults + Name+ '.tif' ) , SizedMask.astype('uint16')) 
     print('Stardist segmentation on Image')  
     SmartSeeds, ProbabilityMap, StarImage, Markers = STARPrediction3D(gaussian_filter(image,filtersize), StarModel,  n_tiles, MaskImage = Mask, UseProbability = UseProbability, smartcorrection = smartcorrection, globalthreshold = globalthreshold, min_size = min_size, extent = extent, seedpool = seedpool)
-    SmartSeeds= remove_small_objects(SmartSeeds.astype('uint16'), min_size = min_size)
-    SmartSeeds= remove_big_objects(SmartSeeds.astype('uint16'), max_size = max_size)
+   
+    for i in range(0, SmartSeeds.shape[0]):
+       SmartSeeds[i,:] = remove_small_objects(SmartSeeds[i,:].astype('uint16'), min_size = min_size)
+       SmartSeeds[i,:] = remove_big_objects(SmartSeeds[i,:].astype('uint16'), max_size = max_size)
     SmartSeeds = fill_label_holes(SmartSeeds.astype('uint16'))
+    if startZ > 0:
+         SmartSeeds[0:startZ,:,:] = 0
     SmartSeeds = RemoveLabels(SmartSeeds) 
     SizedSmartSeeds[:, :SmartSeeds.shape[1], :SmartSeeds.shape[2]] = SmartSeeds
-    SizedProbabilityMap[:, :ProbabilityMap.shape[1], :ProbabilityMap.shape[2]] = ProbabilityMap        
+    SizedProbabilityMap[:, :ProbabilityMap.shape[1], :ProbabilityMap.shape[2]] = ProbabilityMap           
+    
     imwrite((StarDistResults + Name+ '.tif' ) , StarImage.astype('uint16'))
     imwrite((SmartSeedsResults + Name+ '.tif' ) , SizedSmartSeeds.astype('uint16'))
     imwrite((ProbabilityResults + Name+ '.tif' ) , ProbabilityMap.astype('float32'))
@@ -797,7 +825,7 @@ def Conditioncheck(centroid, boxA, p, ndim, extent):
 def WatershedwithMask3D(Image, Label,mask, grid, extent = 0, seedpool = True): 
     properties = measure.regionprops(Label, Image) 
     binaryproperties = measure.regionprops(label(mask), Image) 
-    
+    print(seedpool)
     
     Coordinates = [prop.centroid for prop in properties] 
     BinaryCoordinates = [prop.centroid for prop in binaryproperties]
@@ -825,10 +853,9 @@ def WatershedwithMask3D(Image, Label,mask, grid, extent = 0, seedpool = True):
     markers_raw = np.zeros_like(Image) 
     markers_raw[tuple(coordinates_int.T)] = 1 + np.arange(len(Coordinates)) 
     markers = morphology.dilation(markers_raw.astype('uint16'), morphology.ball(2))
+    mask = np.logical_or(mask, Label > 0)
+    watershedImage = watershed(-Image, markers, mask = mask.copy() )
     
-    watershedImage = watershed(-Image, markers, mask = mask.copy()) 
-
-
     
     return watershedImage, markers
 
@@ -1204,4 +1231,4 @@ def axes_dict(axes):
     """
     axes, allowed = axes_check_and_normalize(axes,return_allowed=True)
     return { a: None if axes.find(a) == -1 else axes.find(a) for a in allowed }
-    # return collections.namedt     
\ No newline at end of file
+    # return collections.namedt     
